#include <stdio.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


void set_dtr(int fd, int dtr)
{
    int ser;
    ioctl(fd, TIOCMGET, &ser);
    if(dtr)
        ser |= TIOCM_DTR;
    else
        ser &= ~TIOCM_DTR;
    ioctl(fd, TIOCMSET, &ser);
}

int main(int argc, const char *const * argv)
{
    char buf[1024];
    int fd;
    struct termios tios;

    if(argc < 2) {
        fprintf(stderr, "Com port not specified\n");
        return 1;
    }
    
    fd = open(argv[1], O_RDWR);
    if(fd == -1) {
        perror(argv[1]);
        return 1;
    }

    if(-1 == tcgetattr(fd, &tios)) {
        perror("tcgetattr");
        return 1;
    }
    cfsetspeed(&tios, B19200);
    if(-1 == tcsetattr(fd, TCSANOW, &tios)) {
        perror("tcsetattr");
        return 1;
    }
    set_dtr(fd, 1);

    for(;;) {
        int rc;
        fd_set rset;
        FD_ZERO(&rset);
        FD_SET(0, &rset);        
        FD_SET(fd, &rset);        
        rc = select(fd+1, &rset, NULL, NULL, NULL);
        if(rc == -1) {
            perror("select");
            continue;
        }
        if(FD_ISSET(fd, &rset)) {
            rc = read(fd, buf, sizeof(buf));
            write(1, buf, rc);
            continue;
        }
        if(FD_ISSET(0, &rset)) {
            rc = read(0, buf, sizeof(buf));
            if(rc == 0) { /* EOF!!! */
                return 0;
            }
            set_dtr(fd, 0);
            write(fd, buf, rc-1);
            write(fd, "\r\n", 2);
            tcdrain(fd);
            usleep(20000 / 192 * (rc+1)*8  /*2000000*/);
            set_dtr(fd, 1);
        }
    }
    return 0;
}
