#include <avr/io.h> 
#include <avr/interrupt.h>  /* for sei() */

#include <util/delay.h>     /* for _delay_ms() */

#include "uart.h"

#ifndef F_CPU
#define F_CPU 12000000UL
#endif

#define UART_BAUD_RATE      19200
#define UART_SOME_ERROR \
    (UART_NO_DATA|UART_FRAME_ERROR|UART_OVERRUN_ERROR|UART_BUFFER_OVERFLOW)


#define LED_ON() (PORTB&=~(1<<2))
#define LED_OFF() (PORTB|=(1<<2))
#define LED_SWITCH() (PORTB^=(1<<2))
#define LED_IS_ON() (!(PORTB & (1<<2)))

#define OPORT2 PORTB
#define OUBIT2_A 1
#define OUBIT2_B 3
#define OUBIT2_C 4
#define OUBIT2_D 5

#define DIR_OUTPUT() (OPORT2 |= (1<<1))
#define DIR_INPUT() (OPORT2 &= ~(1<<1))

uint8_t buffer[128];

void main() __attribute__ ((noreturn));
void main()
{
    uint16_t c;
    uint8_t buf_idx = 0;
    uint8_t i;

    LED_OFF();
    DDRB|=1<<2;
    DDRB|=0x3a;


    sei();
    DIR_INPUT();

    _delay_ms(500);
    LED_ON();
    _delay_ms(500);
    LED_OFF();

    _delay_ms(500);
    LED_ON();
    _delay_ms(500);
    LED_OFF();

    uart_init(UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU)); 

    for(;;) {
        if(((c=uart_getc()) & UART_SOME_ERROR) == 0) {
            buffer[buf_idx++] = c;
            if(buf_idx >= sizeof(buffer))
                buf_idx = 0;
#if 0
            LED_SWITCH();
            _delay_ms(50);
#endif
            if(c == 13 || c == 10) {
                _delay_ms(500);
                LED_ON();
                _delay_ms(500);
                LED_OFF();
                _delay_ms(1000);
                DIR_OUTPUT();
                LED_ON();
                for(i = 0; i < buf_idx; i++) {
                    uart_putc(buffer[i]);
                }
                buf_idx = 0;
                _delay_ms(500);
                DIR_INPUT();
                LED_OFF();
            }
        }
    }
}
