/*
 * UNC001 host-side software. (c) Unicontrollers, Ltd., 2010-2013
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file unc_data.h
 *  
 *
 * Written by Andrey V. Stolyarov
 *
 */

#ifndef UNC_DATA_H_SENTRY
#define UNC_DATA_H_SENTRY

#include <stdint.h>

#define UNC_INPUTS_COUNT 8



struct unc001_global_configuration_str {
    uint8_t  req_id;
    uint8_t  reserved[3];
    uint32_t version;
    uint32_t unused1[2];
		/* from here, the EEPROM-stored part (48 bytes) begins */
    union {
        uint32_t hardware_config;
        uint8_t  hardware_config_bytes[4];
    };
    union {
        uint32_t user_config;
        uint8_t  user_config_bytes[4];
    };
    uint32_t unused2[2];
    union {
        uint32_t reactions[UNC_INPUTS_COUNT];
        uint8_t  react_bytes[UNC_INPUTS_COUNT][4];
    };
};

struct unc001_global_state_str {
    uint8_t  req_id;
    uint8_t  reserved[3];
    uint16_t counters[UNC_INPUTS_COUNT];
    union {
        uint16_t state_bits;
        uint8_t  state_bits_bytes[2];
    };
    union {
        uint16_t fsm_state;
        uint8_t  fsm_state_bytes[2];
    };
    union {
        uint32_t timemark;
        uint8_t  timemark_bytes[4];
        uint16_t timemark_words[2];
    };
    uint16_t reserved2;
    uint8_t  buf_start;
    uint8_t  buf_end;
    uint8_t  message_buffer[32];
};

struct unc001_extra_state_str_common {
    uint8_t req_id;
    uint8_t page_seq;      /* (page_seq & 0x80) means more pages to come */
    uint8_t page_version;  /* 0 for the legacy format, etc, see below */
    uint8_t reserved1;
    uint8_t data[60];
};

struct unc001_extra_state_str_0 { /* legacy format */
    uint8_t req_id;
    uint8_t page_seq;      /* renamed; was always 0 in older versions */
    uint8_t page_version;  /* renamed; was always 0 in older versions */
    uint8_t result;
    uint8_t data_buffer[20];
    int16_t ow_temperature[4];
    uint8_t ow_ids[4][8];
};

struct unc001_extra_state_str_1 { /* vers. 3 main format */
    uint8_t req_id;
    uint8_t page_seq;
    uint8_t page_version;
    uint8_t reserved1;
    union {
        int16_t ow_temperature[6];
        uint16_t ow_data_words[6];
        uint8_t ow_data_bytes[6][2];
    };
    uint8_t ow_ids[6][8];
};

#endif
